# -*- coding: utf-8 -*-
# Copyright © 2015 Polyconseil SAS
# SPDX-License-Identifier: BSD-3-Clause
#

import sys

PY_VERSION = sys.version_info[0]

IS_PY3 = PY_VERSION == 3
IS_PY2 = not IS_PY3


def int_from_bytes(data, byteorder, signed=False):
    """Convert bytes to an integer

    Args:
        data: bytes, the bytes to convert
        byteorder: 'big' or 'little', indicates the endianness
        signed: indicates whether two’s complement is used to represent the integer.

    """
    if IS_PY3:
        return int.from_bytes(data, byteorder, signed=signed)

    if not len(data):
        return 0
    if byteorder == 'big':
        data = bytearray(reversed(data))
    if isinstance(data, str):
        data = bytearray(data)
    num = 0
    for offset, byte in enumerate(data):
        num += byte << (offset * 8)
    if signed and (data[-1] & 0x80):
        num = num - (2 ** (len(data) * 8))
    return num



# -*- coding: utf-8 -*-
# Copyright © 2015 Polyconseil SAS
# SPDX-License-Identifier: BSD-3-Clause
#

from .decode import (
    MESSAGE_MAX_LENGTH, message_decode,
    message_get_current_multiplexing_mode, message_get_signal, signal_decode)
from .exceptions import (
    CanetonError, DecodingError, InvalidBitStart, InvalidDBC,
    MessageNotFound)

__all__ = [
    'MESSAGE_MAX_LENGTH',
    'message_decode', 'message_get_current_multiplexing_mode',
    'message_get_signal', 'signal_decode',
    'CanetonError', 'DecodingError', 'InvalidBitStart', 'InvalidDBC', 'MessageNotFound'
]



# -*- coding: utf-8 -*-
# Copyright © 2015 Polyconseil SAS
# SPDX-License-Identifier: BSD-3-Clause
#

from unittest import TestCase
import binascii
import json

import caneton


class TestDecode(TestCase):

    def test_signal_with_empty_value(self):
        signal_info = {
            'bit_start': 32,
            'length': 16,
            'little_endian': 1,
        }
        message_binary_lsb = '0000000010101111000000001010001100000000'
        # message_binary_msb is not required (LSB)
        with self.assertRaises(caneton.DecodingError) as cm:
            caneton.signal_decode(
                "Foo", signal_info, None, message_binary_lsb, len(message_binary_lsb))
        exception = cm.exception
        self.assertEqual(
            str(exception),
            "The string value extracted for signal 'Foo' is empty [-8:8].")

    def test_message(self):
        message_id = 0x701
        message_data = binascii.unhexlify('01780178010000')
        message_length = len(message_data)

        with open('./tests/dbc.json', 'r') as f:
            dbc_json = json.loads(f.read())

        message = caneton.message_decode(
            message_id=message_id, message_length=message_length,
            message_data=message_data, dbc_json=dbc_json)
        signal = caneton.message_get_signal(message, 'Bar2')
        self.assertEqual(signal['name'], 'Bar2')
        self.assertEqual(signal['value'], 188.0)
        self.assertEqual(signal['unit'], 'V')

    def test_message_wo_signals(self):
        message_id = 542
        message_length = 8
        message_data = b' ' * message_length

        with open('./tests/dbc.json', 'r') as f:
            dbc_json = json.loads(f.read())

        message = caneton.message_decode(
            message_id=message_id, message_length=message_length,
            message_data=message_data, dbc_json=dbc_json)
        self.assertEqual(message['signals'], [])
        self.assertEqual(
            caneton.message_get_signal(message, 'Doesntexist'), None)



# -*- coding: utf-8 -*-
# Copyright © 2015 Polyconseil SAS
# SPDX-License-Identifier: BSD-3-Clause
#

from unittest import TestCase

import caneton.compat


class CompatTestCase(TestCase):

    def test_from_bytes(self):
        self.assertEqual(caneton.compat.int_from_bytes(b'\x00\x10', byteorder='big'), 16)

    def test_from_bytes_little_endian(self):
        self.assertEqual(caneton.compat.int_from_bytes(b'\x00\x10', byteorder='little'), 4096)

    def test_int_from_bytes_negative(self):
        self.assertEqual(caneton.compat.int_from_bytes(b'\xfc\x00', byteorder='big', signed=True), -1024)

    def test_int_from_bytes_signed(self):
        self.assertEqual(caneton.compat.int_from_bytes(b'\xfc\x00', byteorder='big', signed=False), 64512)

    def test_int_from_bytes_iterable(self):
        self.assertEqual(caneton.compat.int_from_bytes([255, 0, 0], byteorder='big'), 16711680)

    def test_from_bytes_cpython(self):
        # test suite taken from cpython code
        # https://hg.python.org/cpython/file/3.4/Lib/test/test_long.py#l1075

        def check(tests, byteorder, signed=False):
            for test, expected in tests.items():
                try:
                    self.assertEqual(
                        caneton.compat.int_from_bytes(test, byteorder, signed=signed),
                        expected
                    )
                except Exception:
                    raise AssertionError(
                        "failed to convert {0} with byteorder={1!r} and signed={2}"
                        .format(test, byteorder, signed)
                    )

        # Convert signed big-endian byte arrays to integers.
        tests1 = {
            b'': 0,
            b'\x00': 0,
            b'\x00\x00': 0,
            b'\x01': 1,
            b'\x00\x01': 1,
            b'\xff': -1,
            b'\xff\xff': -1,
            b'\x81': -127,
            b'\x80': -128,
            b'\xff\x7f': -129,
            b'\x7f': 127,
            b'\x00\x81': 129,
            b'\xff\x01': -255,
            b'\xff\x00': -256,
            b'\x00\xff': 255,
            b'\x01\x00': 256,
            b'\x7f\xff': 32767,
            b'\x80\x00': -32768,
            b'\x00\xff\xff': 65535,
            b'\xff\x00\x00': -65536,
            b'\x80\x00\x00': -8388608
        }
        check(tests1, 'big', signed=True)

        # Convert signed little-endian byte arrays to integers.
        tests2 = {
            b'': 0,
            b'\x00': 0,
            b'\x00\x00': 0,
            b'\x01': 1,
            b'\x00\x01': 256,
            b'\xff': -1,
            b'\xff\xff': -1,
            b'\x81': -127,
            b'\x80': -128,
            b'\x7f\xff': -129,
            b'\x7f': 127,
            b'\x81\x00': 129,
            b'\x01\xff': -255,
            b'\x00\xff': -256,
            b'\xff\x00': 255,
            b'\x00\x01': 256,
            b'\xff\x7f': 32767,
            b'\x00\x80': -32768,
            b'\xff\xff\x00': 65535,
            b'\x00\x00\xff': -65536,
            b'\x00\x00\x80': -8388608
        }
        check(tests2, 'little', signed=True)

        # Convert unsigned big-endian byte arrays to integers.
        tests3 = {
            b'': 0,
            b'\x00': 0,
            b'\x01': 1,
            b'\x7f': 127,
            b'\x80': 128,
            b'\xff': 255,
            b'\x01\x00': 256,
            b'\x7f\xff': 32767,
            b'\x80\x00': 32768,
            b'\xff\xff': 65535,
            b'\x01\x00\x00': 65536,
        }
        check(tests3, 'big', signed=False)

        # Convert integers to unsigned little-endian byte arrays.
        tests4 = {
            b'': 0,
            b'\x00': 0,
            b'\x01': 1,
            b'\x7f': 127,
            b'\x80': 128,
            b'\xff': 255,
            b'\x00\x01': 256,
            b'\xff\x7f': 32767,
            b'\x00\x80': 32768,
            b'\xff\xff': 65535,
            b'\x00\x00\x01': 65536,
        }
        check(tests4, 'little', signed=False)
